# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 Thomas Hearing <thomas.hearing@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gettext
from gettext import gettext as _
gettext.textdomain('wikitron')

from gi.repository import Gtk, WebKit, Unity, Dbusmenu # pylint: disable=E0611
import logging
logger = logging.getLogger('wikitron')

from wikitron_lib import Window
from wikitron.AboutWikitronDialog import AboutWikitronDialog
from wikitron.PreferencesWikitronDialog import PreferencesWikitronDialog

import time
from math import floor

import BookmarksDialog
import hconstants
# See wikitron_lib.Window.py for more details about how this class works
class WikitronWindow(Window):
    __gtype_name__ = "WikitronWindow"
    
    wiki_base_url="http://en.wikipedia.org"
    wiki_api_url="http://en.wikipedia.org/w/api.php"
    main_webview=0
    main_scrollview=0
    mbs_win=0
    b_back=0
    b_forward=0
    b_addbookmark=0
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(WikitronWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutWikitronDialog
        self.PreferencesDialog = PreferencesWikitronDialog

        # Code for other initialization actions should be added here.
        self.pdata_init()
        self.cache_init()
        self.connect("delete-event",self.closing_handler)
        self.main_webview=WebKit.WebView()
        self.apply_config()
        self.main_scrollview=builder.get_object("main_scroll")
        self.b_back=builder.get_object("back_button")
        self.b_forward=builder.get_object("forward_button")
        self.b_addbookmark=builder.get_object("addbookmark_button")
        self.main_scrollview.add(self.main_webview)
        self.show_all()
        
        self.unity_init()        
        
        self.main_webview.connect("navigation-policy-decision-requested",self.main_policy_handler)
        self.open_page_from_title("Main Page",0)
        
    def apply_config(self):
        #Apply some setting to the webview to enhace viewing
        cs=self.main_webview.get_settings()
        cs.set_property("enable-default-context-menu",False)
        cs.set_property("enable-page-cache",True)
        cs.set_property("enable-scripts",False)
        self.main_webview.set_settings(cs)
        
    def api_query(self,params):
        try:
            import urllib2, json
            apiurl=self.wiki_api_url+"?"
            for i,d in params.iteritems():
                apiurl+=i+"="+urllib2.quote(d)+"&"
            apiurl=apiurl[:-1]
            req=urllib2.Request(apiurl)
            #Add a url
            req.add_header("User-Agent","WikiTron")
            reply=urllib2.urlopen(req)
            replytext=reply.read()
            reply.close()
            #print "\n\n"+replytext+"\n\n"
            return json.loads(replytext)
        except:
            return False
        
    def open_page_from_title(self,title,hm):
        import hashlib
        from os import path
        if title.lower() in self.cache_index["contents"]:
            #print "Checking LOAD"
            #print ""
            if (self.cache_index["contents"][title.lower()]["time"]+(60*60*6)) > time.time():
                #print "LOAD: cache"
                #recent copy in cache. load from there
                try:
                    filename=self.cache_index["contents"][title.lower()]["filename"]
                    filepath=path.join(self.cache_folder,filename)
                    #print "1"
                    f=open(filepath)
                    text=f.read()
                    f.close()
                    #print "2"
                    self.load_page(self.cache_index["contents"][title.lower()]["title"],text,hm)
                    #print "LOAD ok"
                    return
                except:
                    #print "LOAD error from cache"
                    pass
            
        #print "LOAD: online"
        #Request the page
        rep=self.api_query({"action":"parse","format":"json","page":title,"prop":"text"})
        if rep == False:
            self.load_error_page(title,rep["error"]["info"],hm)
        elif "error" in rep:
            self.load_error_page(title,rep["error"]["info"],hm)
        else:
            ptitle=rep["parse"]["title"]
            ptext=rep["parse"]["text"]["*"]
            
            #Store the page in the cache
            m=hashlib.md5()
            m.update(ptitle)
            filename=m.hexdigest()
            filepath=path.join(self.cache_folder,filename)
            try:
                import codecs
                f=codecs.open(filepath,encoding="utf-8",mode="w")
                f.write(ptext)
                f.close()
                self.cache_index["contents"][ptitle.lower()]={"title":ptitle,"time":floor(time.time()),"type":"standard","filename":filename}
            except:
                #Cache isn't working, but we should be OK
                pass
            #Display the page
            self.load_page(ptitle,ptext,hm)
        
    def load_error_page(self,title,message,hm):
        self.set_title("Error Loading Page - WikiTron")
        self.main_webview.load_html_string(hconstants.eheader+message+hconstants.efooter,self.wiki_base_url)
        
        #Update the history
        self.pdata["history"].append({"time":time.time(), "page":title})
        
        #Update the history
        if hm == (-1):
            #Going back
            self.session_history_place-=1
        elif hm == 1:
            #Going forward
            self.session_history_place+=1
        else:
            #Normal navigation
            self.session_history=self.session_history[:self.session_history_place-1]
            self.session_history.insert(self.session_history_place,title)
            self.session_history_place+=1
            
        self.update_back_forward_sens()
        
        self.current_title=""
        self.b_addbookmark.set_sensitive(False)
        
    def load_page(self,title,text,hm):
        self.set_title(title+" - WikiTron")
        #+"<h1>"+title+"</h1>\n"+
        self.main_webview.load_html_string(hconstants.header+text+hconstants.footer,self.wiki_base_url)
        
        #Update the history
        self.pdata["history"].append({"time":time.time(), "page":title})
        
        #Update the session history
        #print "LOADING: "+title
        #print "\n\nBEFORE:\n"+str(self.session_history_place)+"\n"+str(self.session_history)
        if hm == (-1):
            #Going back
            self.session_history_place-=1
        elif hm == 1:
            #Going forward
            self.session_history_place+=1
        else:
            #Normal navigation
            #print "Start with: "+str(self.session_history)
            self.session_history=self.session_history[:self.session_history_place-1]
            #print "Now: "+str(self.session_history)
            self.session_history.insert(self.session_history_place,title)
            #print "And after insert: "+str(self.session_history)
            self.session_history_place+=1
            
        #print "\n\nAFTER:\n"+str(self.session_history_place)+"\n"+str(self.session_history)
        self.update_back_forward_sens()
        
        self.current_title=title
        if self.current_title in self.pdata["bookmarks"]:
            self.b_addbookmark.set_sensitive(False)
        else:
            self.b_addbookmark.set_sensitive(True)
        
        
    def update_back_forward_sens(self):
        if self.session_history_place>2:
            self.b_back.set_sensitive(True)
        else:
            self.b_back.set_sensitive(False)
        if self.session_history_place<=len(self.session_history):
            self.b_forward.set_sensitive(True)
        else:
            self.b_forward.set_sensitive(False)
        
    def main_policy_handler(self,webview,webframe,networkrequest,webnavigationaction,webpolicydecision):
        target_url=webnavigationaction.get_original_uri()
        #Check if its on the wiki
        if target_url.startswith(self.wiki_base_url):
            #Now check what type of link it is on the wiki
            remainder=target_url[len(self.wiki_base_url):]
            if ":" in remainder:
                #Probably some special page - open in browser, to be safe
                import webbrowser
                webbrowser.open(target_url)
                return True
            elif remainder.startswith("/wiki/"):
                #Looks like a wiki page
                try:
                    self.open_page_from_title(remainder[6:].replace("_"," "),0)
                except:
                    pass
                return True
            #Not sure what it is
            return False
        #Assume its not on the wiki, and open it in the default browser
        import webbrowser
        webbrowser.open(target_url)
        return True
    
    def closing_handler(self,a,b):
        self.cache_close()
        self.pdata_close()
    
    cache_index={}
    cache_index_file=""
    cache_folder=""
    def cache_init(self):
        from gi.repository import GLib
        import os, json
        self.cache_folder=os.path.join(GLib.get_user_cache_dir(),"wikitron")
        if not os.path.exists(self.cache_folder):
            os.mkdir(self.cache_folder,0700)
        self.cache_index_file=os.path.join(self.cache_folder,"index.json")
        if os.path.exists(self.cache_index_file):
            f=open(self.cache_index_file)
            self.cache_index=json.load(f)
            f.close()
        else:
            self.cache_index={"version":1, "contents":{}}
            
    def cache_close(self):
        import json, os
        if not os.path.exists(self.cache_folder):
            os.mkdir(self.cache_folder,0700)
        f=open(self.cache_index_file,"w")
        json.dump(self.cache_index,f)
        f.close()
        
    def empty_cache(self,a):
        import os
        for i,d in self.cache_index["contents"].iteritems():
            filename=self.cache_index["contents"][i]["filename"]
            filepath=os.path.join(self.cache_folder,filename)
            os.remove(filepath)
        del self.cache_index["contents"]
        self.cache_index["contents"]={}
    
    def search_activated(self,button_obj):
        self.open_page_from_title(button_obj.get_text(),0)
    
    def search_image_activated(self,button_obj,a,b):
        self.open_page_from_title(button_obj.get_text(),0)
        
    def hbs_show(self,a):
        self.mbs_win=BookmarksDialog.BookmarksDialog()
        self.mbs_win.run()
    
    session_history=[]
    session_history_place=1
    
    def m_quit(self,a):
        #Probably shouldn't call it like this
        self.closing_handler(0,0)
        self.destroy()
    
    def m_copy(self,a):
        self.main_webview.copy_clipboard()
    
    def back_clicked(self,a):
        target=self.session_history[self.session_history_place-3]
        if type(target) is dict:
            self.load_special_page(target["Spec"],-1)
        else:
            self.open_page_from_title(target,-1)
        
    def forward_clicked(self,b):
        target=self.session_history[self.session_history_place-1]
        if type(target) is dict:
            self.load_special_page(target["Spec"],1)
        else:
            self.open_page_from_title(target,1)
        
    def unity_init(self):
        ul=Unity.LauncherEntry.get_for_desktop_id ("extras-wikitron.desktop")
        
        l=Dbusmenu.Menuitem.new()
        i_mp=Dbusmenu.Menuitem.new()
        i_mp.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Main Page")
        i_mp.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
        i_mp.connect("item-activated",self.unity_main_page)
        l.child_append(i_mp)
        
        i_b=Dbusmenu.Menuitem.new()
        i_b.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "Bookmarks")
        i_b.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
        i_b.connect("item-activated",self.unity_bookmarks)
        l.child_append(i_b)
        
        i_h=Dbusmenu.Menuitem.new()
        i_h.property_set (Dbusmenu.MENUITEM_PROP_LABEL, "History")
        i_h.property_set_bool (Dbusmenu.MENUITEM_PROP_VISIBLE, True)
        i_h.connect("item-activated",self.unity_history)
        l.child_append(i_h)
        
        ul.set_property("quicklist", l)
        
    def unity_main_page(self,a,b):
        self.open_page_from_title("Main Page",0)
        
    def unity_bookmarks(self,a,b):
        self.load_special_page("Bookmarks",0)
        
    def unity_history(self,a,b):
        self.load_special_page("History",0)
        
    pdata={}
    pdata_file=""
    pdata_folder=""
    def pdata_init(self):
        from gi.repository import GLib
        import os, json
        self.pdata_folder=os.path.join(GLib.get_user_config_dir(),"wikitron")
        if not os.path.exists(self.pdata_folder):
            os.mkdir(self.pdata_folder,0700)
        self.pdata_file=os.path.join(self.pdata_folder,"pdata.json")
        if os.path.exists(self.pdata_file):
            f=open(self.pdata_file)
            self.pdata=json.load(f)
            f.close()
        else:
            self.pdata={"version":1, "history":[], "bookmarks":[]}
            
    def pdata_close(self):
        import json, os
        if not os.path.exists(self.pdata_folder):
            os.mkdir(self.pdata_folder,0700)
        f=open(self.pdata_file,"w")
        json.dump(self.pdata,f)
        f.close()
    
    current_title=""
    
    def addbookmark_clicked(self,a):
        if not self.current_title=="":
            if not self.current_title in self.pdata["bookmarks"]:
                self.pdata["bookmarks"].append(self.current_title)
                self.b_addbookmark.set_sensitive(False)
                
    def load_special_page(self,page,hm):
        if page=="Bookmarks":
            self.pdata["bookmarks"].sort()
            content="<ul>\n"
            for i in self.pdata["bookmarks"]:
                content+="<li><a href=\""+self.wiki_base_url+"/wiki/"+i.replace(" ","_")+"\" title=\""+i+"\">"+i+"</a></li>\n"
            content+="</ul>"
        elif page=="History":
            dl=sorted(self.pdata["history"],key=lambda a:a["time"],reverse=True)
            content="<ul>\n"
            for i in dl:
                content+="<li><a href=\""+self.wiki_base_url+"/wiki/"+i["page"].replace(" ","_")+"\" title=\""+i["page"]+"\">"+i["page"]+"</a> <em>(Accessed on "+time.strftime("%A, %d of %B %Y at %I:%M:%S%p",time.localtime(i["time"]))+")</em></li>\n"
            content+="</ul>"
        else:
            content="Error"
        #print "\n\n"
        #print content
        #print "\n\n"
        content=hconstants.special_page.replace("!TITLE!",page,1).replace("!CONTENTS!",content,1)
        
        self.set_title(page+" - WikiTron")
        self.main_webview.load_html_string(content,self.wiki_base_url)
        
        #Update the history
        if hm == (-1):
            #Going back
            self.session_history_place-=1
        elif hm == 1:
            #Going forward
            self.session_history_place+=1
        else:
            #Normal navigation
            self.session_history=self.session_history[:self.session_history_place-1]
            self.session_history.insert(self.session_history_place,{"Spec":page})
            self.session_history_place+=1
            
        self.update_back_forward_sens()
        
        self.current_title=""
        self.b_addbookmark.set_sensitive(False)
    
    def show_bookmarks(self,a):
        self.load_special_page("Bookmarks",0)
    
    def show_history(self,a):
        self.load_special_page("History",0)
